@@ -1,8 +1,8 @@
-/*  $Id: AfternoonStalkerEngine.h,v 1.35 2009/02/27 03:08:33 sarrazip Exp $
+/*  $Id: AfternoonStalkerEngine.h,v 1.45 2017/12/09 18:31:57 sarrazip Exp $
     AfternoonStalkerEngine.h - A robot-killing video game engine.
 
     afternoonstalker - A robot-killing video game.
-    Copyright (C) 2001-2009 Pierre Sarrazin <http://sarrazip.com/>
+    Copyright (C) 2001-2012 Pierre Sarrazin <http://sarrazip.com/>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-    02111-1307, USA.
+    You should have received a copy of the GNU General Public
+    License along with this program; if not, write to the Free
+    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA  02110-1301, USA.
 */
 
 #ifndef _H_AfternoonStalkerEngine
@@ -30,7 +30,6 @@
 #include <flatzebra/GameEngine.h>
 #include <flatzebra/Sprite.h>
 #include <flatzebra/SoundMixer.h>
-#include <flatzebra/KeyState.h>
 
 class RobotSprite;
 
@@ -48,9 +47,9 @@ class AfternoonStalkerEngine : public GameEngine
 public:
 
     enum { RIGHT, UP, LEFT, DOWN };
-    /*	Possible directions of movement, in trigonometric order.
-	This enumeration is garanteed to start at zero.
-	The order of the constants is guaranteed, so they are 0, 1, 2, 3.
+    /*  Possible directions of movement, in trigonometric order.
+        This enumeration is garanteed to start at zero.
+        The order of the constants is guaranteed, so they are 0, 1, 2, 3.
     */
 
     enum Rules { AS_RULES_1, AS_RULES_2 };
@@ -59,21 +58,23 @@ public:
     */
 
     AfternoonStalkerEngine(const std::string &windowManagerCaption,
-			    bool fullScreen,
-			    const std::string &mazeFilename,
-			    Rules rules)
-				throw(int, std::string);
+                            bool _useSound,
+                            bool fullScreen,
+                            size_t _mazeIndex,
+                            Rules rules,
+                            bool processActiveEvent);
     /*  See base class.
 
-	'windowManagerCaption' must contain the title to give to the window.
-	'useSound' determines if sound effects are produced.
-	'fullScreen' attempts to display the fame in full screen mode
-	instead of using an ordinary window.
-	'onlyBlueRobots' sets a practice mode where only blue robots
-	are sent out (they are smarter than gray robots, but only
-	require one bullet to be killed).
+        'windowManagerCaption' must contain the title to give to the window.
+        'fullScreen' attempts to display the fame in full screen mode
+        instead of using an ordinary window.
+        '_mazeIndex' must be a zero-based index of a .set file available
+        in the game's PKGDATADIR.
+        'rules': see manual page (doc/afternoonstalker.6.in).
+        'processActiveEvent' determines if the game automatically pauses
+        when the game window loses focus.
 
-	Throws a string or an integer code if an error occurs.
+        Throws a string or an integer code if an error occurs.
     */
 
     virtual ~AfternoonStalkerEngine();
@@ -87,46 +88,43 @@ public:
     /*  Inherited.
     */
 
+    virtual void processActivation(bool appActive);
+    /*  Inherited.
+    */
+
     virtual bool tick();
     /*  Inherited.
     */
 
 private:
 
-    ///////////////////////////////////////////////////////////////////////////
-    //
-    //  LOCAL TYPES, CLASSES AND CONSTANTS
-    //
-    //
-
-
     enum TileNo
     {
-	// The first value must be zero
-	WALL_TILE,
+        // The first value must be zero
+        WALL_TILE,
 
-	// The CORNER* and WALLEND* names must be contiguous
-	CORNER0_TILE,
-	CORNER1_TILE,
-	CORNER2_TILE,
-	CORNER3_TILE,
-	WALLEND0_TILE,
-	WALLEND1_TILE,
-	WALLEND2_TILE,
-	WALLEND3_TILE,
+        // The CORNER* and WALLEND* names must be contiguous
+        CORNER0_TILE,
+        CORNER1_TILE,
+        CORNER2_TILE,
+        CORNER3_TILE,
+        WALLEND0_TILE,
+        WALLEND1_TILE,
+        WALLEND2_TILE,
+        WALLEND3_TILE,
 
-	FLOOR_TILE,
-	COBWEB_TILE,
+        FLOOR_TILE,
+        COBWEB_TILE,
 
-	BUNKER_TILE,
+        BUNKER_TILE,
 
-	// The FLOOR[0-3]* names must be contiguous
-	BUNKER0_TILE,
-	BUNKER1_TILE,
-	BUNKER2_TILE,
-	BUNKER3_TILE,
+        // The FLOOR[0-3]* names must be contiguous
+        BUNKER0_TILE,
+        BUNKER1_TILE,
+        BUNKER2_TILE,
+        BUNKER3_TILE,
 
-	BUNKER_DOOR_TILE
+        BUNKER_DOOR_TILE
     };
 
     typedef std::vector<TileNo> TileMatrixRow;
@@ -136,39 +134,43 @@ private:
     enum Privilege
     // Used in positionIsAllowed()
     {
-	PLAYER_PRIV,
-	ENEMY_PRIV,
-	BULLET_PRIV,
-	PIERCING_BULLET_PRIV
+        PLAYER_PRIV,
+        ENEMY_PRIV,
+        BULLET_PRIV,
+        PIERCING_BULLET_PRIV
     };
 
 
     class Menu
     {
     public:
-	Menu() : currentSelectionIndex(0) {}
+        Menu() : title(), entries(), currentSelectionIndex(0), parentMenu(NULL) {}
 
-	void setParentMenu(Menu *p) { parentMenu = p; }
-	Menu *getParentMenu() const { return parentMenu; }
+        void setParentMenu(Menu *p) { parentMenu = p; }
+        Menu *getParentMenu() const { return parentMenu; }
 
-	void setTitle(const std::string &t) { title = t; }
-	const std::string &getTitle() const { return title; }
+        void setTitle(const std::string &t) { title = t; }
+        const std::string &getTitle() const { return title; }
 
-	void addEntry(const std::string &entry) { entries.push_back(entry); }
-	const std::string &getEntry(size_t index) const { return entries[index]; }
+        void addEntry(const std::string &entry) { entries.push_back(entry); }
+        const std::string &getEntry(size_t index) const { return entries[index]; }
 
-	void setSelectionIndex(size_t i) { currentSelectionIndex = i; }
-	void nextSelection() { currentSelectionIndex = (currentSelectionIndex + 1) % entries.size(); }
-	void prevSelection() { currentSelectionIndex = (currentSelectionIndex + entries.size() - 1) % entries.size(); }
-	size_t getSelectionIndex() const { return currentSelectionIndex; }
+        void setSelectionIndex(size_t i) { currentSelectionIndex = i; }
+        void nextSelection() { currentSelectionIndex = (currentSelectionIndex + 1) % entries.size(); }
+        void prevSelection() { currentSelectionIndex = (currentSelectionIndex + entries.size() - 1) % entries.size(); }
+        size_t getSelectionIndex() const { return currentSelectionIndex; }
 
-	size_t getNumEntries() const { return entries.size(); }
+        size_t getNumEntries() const { return entries.size(); }
 
     private:
-	std::string title;
-	std::vector<std::string> entries;
-	size_t currentSelectionIndex;
-	Menu *parentMenu;
+        std::string title;
+        std::vector<std::string> entries;
+        size_t currentSelectionIndex;
+        Menu *parentMenu;
+
+        // Forbidden operations:
+        Menu(const Menu &);
+        Menu operator = (const Menu &);
     };
 
 
@@ -184,7 +186,7 @@ private:
     bool useSound;
     bool soundChunksInitialized;
     Rules rules;
-    std::string mazeFilename;
+    size_t mazeIndex;
 
 
     // SDL color values (see SDL_MapRGB()):
@@ -217,7 +219,7 @@ private:
 
     PixmapArray playerExplosionPA;
     SpriteList playerExplosionSprites;
-	    // when this list is not empty, the player is dying
+            // when this list is not empty, the player is dying
 
     PixmapArray gunPA;
     SpriteList gunSprites;
@@ -225,7 +227,7 @@ private:
 
     /*  ENEMIES:
 
-	CONVENTION: all robot sprites are the same size as the human.
+        CONVENTION: all robot sprites are the same size as the human.
     */
     PixmapArray grayRobotPA;
     PixmapArray blueRobotPA;
@@ -295,37 +297,35 @@ private:
 
 
     ///////////////////////////////////////////////////////////////////////////
-    //
-    //  IMPLEMENTATION FUNCTIONS
-    //
-    //
+
+
     bool animatePlayer();
     int getRobotScore(const Sprite &robot) const;
     void makePlayerShoot(int shotDir);
     Couple attemptMove(const Sprite &s,
-			const bool attemptedDirections[4],
-			int speedFactor) const;
+                        const bool attemptedDirections[4],
+                        int speedFactor) const;
     Couple getSpeedFromAttemptedAndAllowedDirections(
-			const bool attemptedDirections[4],
-			const bool allowedDirections[4],
-			int speedFactor,
-			Couple delta) const;
+                        const bool attemptedDirections[4],
+                        const bool allowedDirections[4],
+                        int speedFactor,
+                        Couple delta) const;
     Couple determineAllowedDirections(const Sprite &s,
-					bool forPlayer,
-					int speedFactor, int tolerance,
-					bool allowedDirections[4]) const;
+                                        bool forPlayer,
+                                        int speedFactor, int tolerance,
+                                        bool allowedDirections[4]) const;
     Couple getDistanceToPerfectPos(const Sprite &s) const;
     bool positionIsAllowed(int direction, Privilege priv,
-				Couple pos, Couple size) const;
+                                Couple pos, Couple size) const;
     std::pair<TileNo, TileNo> getSideTilesAtPos(
-				int direction, Couple pos, Couple size) const;
+                                int direction, Couple pos, Couple size) const;
     TileNo getTileAtPos(Couple pos) const;
     void setTileAtPos(Couple pos, TileNo t);
 
     void animateTemporarySprites(SpriteList &slist) const;
     void animateAutomaticCharacters();
     void killSpritesNearPosition(
-			SpriteList &slist, Couple pos, int minDistance);
+                        SpriteList &slist, Couple pos, int minDistance);
     void moveExplosionSprites(SpriteList &slist);
     bool isOutOfSetting(const Sprite &s) const;
     size_t getNumToughRobots() const;
@@ -344,12 +344,12 @@ private:
     void paralyzePlayer();
     void createRobotExplosion(const RobotSprite &r);
     void createAnimalExplosion(Couple pos,
-    				const PixmapArray &pa, SpriteList &slist);
+                                    const PixmapArray &pa, SpriteList &slist);
     void killEnemy(const Sprite &s);
     void createPlayerExplosion(const Sprite &s);
     void addToScore(long n);
     void createScoreSprites(long n, Couple center);
-    void loadPixmaps() throw(PixmapLoadError);
+    void loadPixmaps();
     void restoreBackground();
     void restoreForeground();
     void putSprite(const Sprite &s);
@@ -358,24 +358,27 @@ private:
     void drawScoreBoard();
     void displayMessage(int row, const char *msg);
 
-    void displayErrorMessage(const std::string &msg) throw();
+    void displayErrorMessage(const std::string &msg);
     void initializeLevel();
     void initializePlayerLife();
     void setTimeUntilNextRobot();
     void setTimeUntilNextAnimal();
-    void doOneTimeInitializations() throw(std::string);
+    void doOneTimeInitializations();
     void initSounds();
     void initMenus();
 
     enum MenuAction { START_GAME, QUIT_GAME, QUIT_PROGRAM, STAY_IN_MENUS };
     MenuAction processMenuMode();
-    void loadLevel() throw(std::string);
+    void loadLevel();
 
     void playSoundEffect(SoundMixer::Chunk &wb);
     void processJoystick();
+    void setPauseMode();
+    void displayCurrentMenu();
+    bool findSecondCobwebTile(size_t &x, size_t &y) const;
 
 
-    /*	Forbidden operations:
+    /*        Forbidden operations:
     */
     AfternoonStalkerEngine(const AfternoonStalkerEngine &);
     AfternoonStalkerEngine &operator = (const AfternoonStalkerEngine &);
